@@ -8,7 +8,7 @@ use crate::pac::pwm0::*;
 use crate::pac::pwm0_ns::*;
 use crate::{
     gpio::{Output, Pin, PushPull},
-    pac::{generic::Reg, Interrupt},
+    pac::Interrupt,
     target_constants::{SRAM_LOWER, SRAM_UPPER},
     time::*,
 };
@@ -129,9 +129,11 @@ where
         }
     }
 
-    /// Sets the associated output pin for the PWM channel and enables it.
+    /// Sets the associated output pin for the PWM channel.
+    ///
+    /// Modifying the pin configuration while the PWM instance is enabled is not recommended.
     #[inline(always)]
-    pub fn set_output_pin(&self, channel: Channel, pin: &Pin<Output<PushPull>>) -> &Self {
+    pub fn set_output_pin(&self, channel: Channel, pin: Pin<Output<PushPull>>) -> &Self {
         self.pwm.psel.out[usize::from(channel)].write(|w| {
             unsafe { w.bits(pin.psel_bits()) };
             w.connect().connected()
@@ -139,6 +141,44 @@ where
         self
     }
 
+    /// Sets the output pin of `channel`, and returns the old pin (if any).
+    ///
+    /// Modifying the pin configuration while the PWM instance is enabled is not recommended.
+    pub fn swap_output_pin(
+        &mut self,
+        channel: Channel,
+        pin: Pin<Output<PushPull>>,
+    ) -> Option<Pin<Output<PushPull>>> {
+        // (needs `&mut self` because it reads, then writes, to the register)
+        let psel = &self.pwm.psel.out[usize::from(channel)];
+        let old = psel.read();
+        let old = if old.connect().is_connected() {
+            unsafe { Some(Pin::from_psel_bits(old.bits())) }
+        } else {
+            None
+        };
+        self.set_output_pin(channel, pin);
+        old
+    }
+
+    /// Disables the output pin of `channel`.
+    ///
+    /// The output pin is returned, if one was previously configured.
+    ///
+    /// Modifying the pin configuration while the PWM instance is enabled is not recommended.
+    pub fn clear_output_pin(&mut self, channel: Channel) -> Option<Pin<Output<PushPull>>> {
+        // (needs `&mut self` because it reads, then writes, to the register)
+        let psel = &self.pwm.psel.out[usize::from(channel)];
+        let old = psel.read();
+        let old = if old.connect().is_connected() {
+            unsafe { Some(Pin::from_psel_bits(old.bits())) }
+        } else {
+            None
+        };
+        psel.reset();
+        old
+    }
+
     /// Enables the PWM generator.
     #[inline(always)]
     pub fn enable(&self) {
@@ -228,7 +268,7 @@ where
         self
     }
 
-    /// Returns selected operating mode of the wave counter.    
+    /// Returns selected operating mode of the wave counter.
     #[inline(always)]
     pub fn counter_mode(&self) -> CounterMode {
         match self.pwm.mode.read().updown().bit() {
@@ -351,7 +391,7 @@ where
         self.start_seq(Seq::Seq0);
     }
 
-    /// Returns duty cycle value for a PWM group.    
+    /// Returns duty cycle value for a PWM group.
     #[inline(always)]
     pub fn duty_on_group(&self, group: Group) -> u16 {
         self.duty_on_value(usize::from(group))
@@ -395,7 +435,7 @@ where
         self.start_seq(Seq::Seq0);
     }
 
-    /// Returns the duty cycle value for a PWM channel.        
+    /// Returns the duty cycle value for a PWM channel.
     #[inline(always)]
     pub fn duty_on(&self, channel: Channel) -> u16 {
         self.duty_on_value(usize::from(channel))
@@ -623,67 +663,67 @@ where
 
     /// Returns reference to `Stopped` event endpoint for PPI.
     #[inline(always)]
-    pub fn event_stopped(&self) -> &Reg<u32, _EVENTS_STOPPED> {
+    pub fn event_stopped(&self) -> &EVENTS_STOPPED {
         &self.pwm.events_stopped
     }
 
     /// Returns reference to `LoopsDone` event endpoint for PPI.
     #[inline(always)]
-    pub fn event_loops_done(&self) -> &Reg<u32, _EVENTS_LOOPSDONE> {
+    pub fn event_loops_done(&self) -> &EVENTS_LOOPSDONE {
         &self.pwm.events_loopsdone
     }
 
     /// Returns reference to `PwmPeriodEnd` event endpoint for PPI.
     #[inline(always)]
-    pub fn event_pwm_period_end(&self) -> &Reg<u32, _EVENTS_PWMPERIODEND> {
+    pub fn event_pwm_period_end(&self) -> &EVENTS_PWMPERIODEND {
         &self.pwm.events_pwmperiodend
     }
 
     /// Returns reference to `Seq0 End` event endpoint for PPI.
     #[inline(always)]
-    pub fn event_seq0_end(&self) -> &Reg<u32, _EVENTS_SEQEND> {
+    pub fn event_seq0_end(&self) -> &EVENTS_SEQEND {
         &self.pwm.events_seqend[0]
     }
 
     /// Returns reference to `Seq1 End` event endpoint for PPI.
     #[inline(always)]
-    pub fn event_seq1_end(&self) -> &Reg<u32, _EVENTS_SEQEND> {
+    pub fn event_seq1_end(&self) -> &EVENTS_SEQEND {
         &self.pwm.events_seqend[1]
     }
 
     /// Returns reference to `Seq0 Started` event endpoint for PPI.
     #[inline(always)]
-    pub fn event_seq0_started(&self) -> &Reg<u32, _EVENTS_SEQSTARTED> {
+    pub fn event_seq0_started(&self) -> &EVENTS_SEQSTARTED {
         &self.pwm.events_seqstarted[0]
     }
 
     /// Returns reference to `Seq1 Started` event endpoint for PPI.
     #[inline(always)]
-    pub fn event_seq1_started(&self) -> &Reg<u32, _EVENTS_SEQSTARTED> {
+    pub fn event_seq1_started(&self) -> &EVENTS_SEQSTARTED {
         &self.pwm.events_seqstarted[1]
     }
 
     /// Returns reference to `Seq0 Start` task endpoint for PPI.
     #[inline(always)]
-    pub fn task_start_seq0(&self) -> &Reg<u32, _TASKS_SEQSTART> {
+    pub fn task_start_seq0(&self) -> &TASKS_SEQSTART {
         &self.pwm.tasks_seqstart[0]
     }
 
     /// Returns reference to `Seq1 Started` task endpoint for PPI.
     #[inline(always)]
-    pub fn task_start_seq1(&self) -> &Reg<u32, _TASKS_SEQSTART> {
+    pub fn task_start_seq1(&self) -> &TASKS_SEQSTART {
         &self.pwm.tasks_seqstart[1]
     }
 
     /// Returns reference to `NextStep` task endpoint for PPI.
     #[inline(always)]
-    pub fn task_next_step(&self) -> &Reg<u32, _TASKS_NEXTSTEP> {
+    pub fn task_next_step(&self) -> &TASKS_NEXTSTEP {
         &self.pwm.tasks_nextstep
     }
 
     /// Returns reference to `Stop` task endpoint for PPI.
     #[inline(always)]
-    pub fn task_stop(&self) -> &Reg<u32, _TASKS_STOP> {
+    pub fn task_stop(&self) -> &TASKS_STOP {
         &self.pwm.tasks_stop
     }
 
