@@ -4,20 +4,18 @@
 
 #[cfg(feature = "9160")]
 use crate::pac::{
-    generic::Reg,
     timer0_ns::{
-        RegisterBlock as RegBlock0, _EVENTS_COMPARE, _TASKS_CAPTURE, _TASKS_CLEAR, _TASKS_COUNT,
-        _TASKS_START, _TASKS_STOP,
+        RegisterBlock as RegBlock0, EVENTS_COMPARE, TASKS_CAPTURE, TASKS_CLEAR, TASKS_COUNT,
+        TASKS_START, TASKS_STOP,
     },
     Interrupt, TIMER0_NS as TIMER0, TIMER1_NS as TIMER1, TIMER2_NS as TIMER2,
 };
 
 #[cfg(not(feature = "9160"))]
 use crate::pac::{
-    generic::Reg,
     timer0::{
-        RegisterBlock as RegBlock0, _EVENTS_COMPARE, _TASKS_CAPTURE, _TASKS_CLEAR, _TASKS_COUNT,
-        _TASKS_START, _TASKS_STOP,
+        RegisterBlock as RegBlock0, EVENTS_COMPARE, TASKS_CAPTURE, TASKS_CLEAR, TASKS_COUNT,
+        TASKS_START, TASKS_STOP,
     },
     Interrupt, TIMER0, TIMER1, TIMER2,
 };
@@ -36,14 +34,14 @@ use crate::pac::{TIMER3, TIMER4};
 // The 832 and 840 expose TIMER3 and TIMER for as timer3::RegisterBlock...
 #[cfg(any(feature = "52832", feature = "52840"))]
 use crate::pac::timer3::{
-    RegisterBlock as RegBlock3, _EVENTS_COMPARE as EventsCompare3, _TASKS_CAPTURE as TasksCapture3,
+    RegisterBlock as RegBlock3, EVENTS_COMPARE as EventsCompare3, TASKS_CAPTURE as TasksCapture3,
 };
 
 // ...but the 833 exposes them as timer0::RegisterBlock. This might be a bug
 // in the PAC, and could be fixed later. For now, it is equivalent anyway.
 #[cfg(feature = "52833")]
 use crate::pac::timer0::{
-    RegisterBlock as RegBlock3, _EVENTS_COMPARE as EventsCompare3, _TASKS_CAPTURE as TasksCapture3,
+    RegisterBlock as RegBlock3, EVENTS_COMPARE as EventsCompare3, TASKS_CAPTURE as TasksCapture3,
 };
 
 use core::marker::PhantomData;
@@ -153,56 +151,56 @@ where
     /// Returns reference to the `START` task endpoint for PPI.
     /// Starts timer.
     #[inline(always)]
-    pub fn task_start(&self) -> &Reg<u32, _TASKS_START> {
+    pub fn task_start(&self) -> &TASKS_START {
         &self.0.as_timer0().tasks_start
     }
 
     /// Returns reference to the `STOP` task endpoint for PPI.
     /// Stops timer.
     #[inline(always)]
-    pub fn task_stop(&self) -> &Reg<u32, _TASKS_STOP> {
+    pub fn task_stop(&self) -> &TASKS_STOP {
         &self.0.as_timer0().tasks_stop
     }
 
     /// Returns reference to the `COUNT` task endpoint for PPI.
     /// Increments timer (counter mode only).
     #[inline(always)]
-    pub fn task_count(&self) -> &Reg<u32, _TASKS_COUNT> {
+    pub fn task_count(&self) -> &TASKS_COUNT {
         &self.0.as_timer0().tasks_count
     }
 
     /// Returns reference to the `CLEAR` task endpoint for PPI.
     /// Clears timer.
     #[inline(always)]
-    pub fn task_clear(&self) -> &Reg<u32, _TASKS_CLEAR> {
+    pub fn task_clear(&self) -> &TASKS_CLEAR {
         &self.0.as_timer0().tasks_clear
     }
 
     /// Returns reference to the CC[0] `CAPTURE` task endpoint for PPI.
     /// Captures timer value to the CC[0] register.
     #[inline(always)]
-    pub fn task_capture_cc0(&self) -> &Reg<u32, _TASKS_CAPTURE> {
+    pub fn task_capture_cc0(&self) -> &TASKS_CAPTURE {
         &self.0.as_timer0().tasks_capture[0]
     }
 
     /// Returns reference to the CC[1] `CAPTURE` task endpoint for PPI.
     /// Captures timer value to the CC[1] register.
     #[inline(always)]
-    pub fn task_capture_cc1(&self) -> &Reg<u32, _TASKS_CAPTURE> {
+    pub fn task_capture_cc1(&self) -> &TASKS_CAPTURE {
         &self.0.as_timer0().tasks_capture[1]
     }
 
     /// Returns reference to the CC[2] `CAPTURE` task endpoint for PPI.
     /// Captures timer value to the CC[2] register.
     #[inline(always)]
-    pub fn task_capture_cc2(&self) -> &Reg<u32, _TASKS_CAPTURE> {
+    pub fn task_capture_cc2(&self) -> &TASKS_CAPTURE {
         &self.0.as_timer0().tasks_capture[2]
     }
 
     /// Returns reference to the CC[3] `CAPTURE` task endpoint for PPI.
     /// Captures timer value to the CC[3] register.
     #[inline(always)]
-    pub fn task_capture_cc3(&self) -> &Reg<u32, _TASKS_CAPTURE> {
+    pub fn task_capture_cc3(&self) -> &TASKS_CAPTURE {
         &self.0.as_timer0().tasks_capture[3]
     }
 
@@ -210,7 +208,7 @@ where
     /// Generated when the counter is incremented and then matches the value
     /// specified in the CC[0] register.
     #[inline(always)]
-    pub fn event_compare_cc0(&self) -> &Reg<u32, _EVENTS_COMPARE> {
+    pub fn event_compare_cc0(&self) -> &EVENTS_COMPARE {
         &self.0.as_timer0().events_compare[0]
     }
 
@@ -218,7 +216,7 @@ where
     /// Generated when the counter is incremented and then matches the value
     /// specified in the CC[1] register.
     #[inline(always)]
-    pub fn event_compare_cc1(&self) -> &Reg<u32, _EVENTS_COMPARE> {
+    pub fn event_compare_cc1(&self) -> &EVENTS_COMPARE {
         &self.0.as_timer0().events_compare[1]
     }
 
@@ -226,7 +224,7 @@ where
     /// Generated when the counter is incremented and then matches the value
     /// specified in the CC[2] register.
     #[inline(always)]
-    pub fn event_compare_cc2(&self) -> &Reg<u32, _EVENTS_COMPARE> {
+    pub fn event_compare_cc2(&self) -> &EVENTS_COMPARE {
         &self.0.as_timer0().events_compare[2]
     }
 
@@ -234,7 +232,7 @@ where
     /// Generated when the counter is incremented and then matches the value
     /// specified in the CC[3] register.
     #[inline(always)]
-    pub fn event_compare_cc3(&self) -> &Reg<u32, _EVENTS_COMPARE> {
+    pub fn event_compare_cc3(&self) -> &EVENTS_COMPARE {
         &self.0.as_timer0().events_compare[3]
     }
 }
@@ -505,35 +503,35 @@ impl Instance for TIMER4 {
 /// Adds task- and event PPI endpoint getters for CC[4] and CC[5] on supported instances.
 #[cfg(any(feature = "52832", feature = "52833", feature = "52840"))]
 pub trait ExtendedCCTimer {
-    fn task_capture_cc4(&self) -> &Reg<u32, TasksCapture3>;
-    fn task_capture_cc5(&self) -> &Reg<u32, TasksCapture3>;
-    fn event_compare_cc4(&self) -> &Reg<u32, EventsCompare3>;
-    fn event_compare_cc5(&self) -> &Reg<u32, EventsCompare3>;
+    fn task_capture_cc4(&self) -> &TasksCapture3;
+    fn task_capture_cc5(&self) -> &TasksCapture3;
+    fn event_compare_cc4(&self) -> &EventsCompare3;
+    fn event_compare_cc5(&self) -> &EventsCompare3;
 }
 
 #[cfg(any(feature = "52832", feature = "52833", feature = "52840"))]
 impl ExtendedCCTimer for Timer<TIMER3> {
     /// Returns reference to the CC[4] `CAPTURE` task endpoint for PPI.
     #[inline(always)]
-    fn task_capture_cc4(&self) -> &Reg<u32, TasksCapture3> {
+    fn task_capture_cc4(&self) -> &TasksCapture3 {
         &self.0.tasks_capture[4]
     }
 
     /// Returns reference to the CC[5] `CAPTURE` task endpoint for PPI.
     #[inline(always)]
-    fn task_capture_cc5(&self) -> &Reg<u32, TasksCapture3> {
+    fn task_capture_cc5(&self) -> &TasksCapture3 {
         &self.0.tasks_capture[5]
     }
 
     /// Returns reference to the CC[4] `COMPARE` event endpoint for PPI.
     #[inline(always)]
-    fn event_compare_cc4(&self) -> &Reg<u32, EventsCompare3> {
+    fn event_compare_cc4(&self) -> &EventsCompare3 {
         &self.0.events_compare[4]
     }
 
     /// Returns reference to the CC[5] `COMPARE` event endpoint for PPI.
     #[inline(always)]
-    fn event_compare_cc5(&self) -> &Reg<u32, EventsCompare3> {
+    fn event_compare_cc5(&self) -> &EventsCompare3 {
         &self.0.events_compare[5]
     }
 }
@@ -542,25 +540,25 @@ impl ExtendedCCTimer for Timer<TIMER3> {
 impl ExtendedCCTimer for Timer<TIMER4> {
     /// Returns reference to the CC[4] `CAPTURE` task endpoint for PPI.
     #[inline(always)]
-    fn task_capture_cc4(&self) -> &Reg<u32, TasksCapture3> {
+    fn task_capture_cc4(&self) -> &TasksCapture3 {
         &self.0.tasks_capture[4]
     }
 
     /// Returns reference to the CC[5] `CAPTURE` task endpoint for PPI.
     #[inline(always)]
-    fn task_capture_cc5(&self) -> &Reg<u32, TasksCapture3> {
+    fn task_capture_cc5(&self) -> &TasksCapture3 {
         &self.0.tasks_capture[5]
     }
 
     /// Returns reference to the CC[4] `COMPARE` event endpoint for PPI.
     #[inline(always)]
-    fn event_compare_cc4(&self) -> &Reg<u32, EventsCompare3> {
+    fn event_compare_cc4(&self) -> &EventsCompare3 {
         &self.0.events_compare[4]
     }
 
     /// Returns reference to the CC[5] `COMPARE` event endpoint for PPI.
     #[inline(always)]
-    fn event_compare_cc5(&self) -> &Reg<u32, EventsCompare3> {
+    fn event_compare_cc5(&self) -> &EventsCompare3 {
         &self.0.events_compare[5]
     }
 }
